@@ -1,7 +1,7 @@
 package Complete::Getopt::Long;
 
-our $DATE = '2016-10-28'; # DATE
-our $VERSION = '0.43'; # VERSION
+our $DATE = '2016-11-04'; # DATE
+our $VERSION = '0.44'; # VERSION
 
 use 5.010001;
 use strict;
@@ -322,9 +322,14 @@ sub complete_cli_arg {
 
     my %seen_opts;
 
-    # for each word, we try to find out whether it's supposed to complete option
-    # name, or option value, or argument, or separator (or more than one of
-    # them). plus some other information.
+    # for each word (each element in this array), we try to find out whether
+    # it's supposed to complete option name, or option value, or argument, or
+    # separator (or more than one of them). plus some other information.
+    #
+    # each element is a hash. if hash contains 'optname' key then it expects an
+    # option name. if hash contains 'optval' key then it expects an option
+    # value.
+
     my @expects;
 
     my $i = -1;
@@ -352,44 +357,72 @@ sub complete_cli_arg {
             {
                 last unless $bundling;
                 my $shorts = $word;
-                if ($shorts =~ s/\A-([^-])(.*)/$2/) {
-                    my $opt = "-$1";
-                    my $opthash = $opts{$opt};
-                    if (!$opthash || $opthash->{parsed}{max_vals}) {
-                        last SPLIT_BUNDLED;
-                    }
-                    $words[$i] = $word = "-$1";
-                    $expects[$i]{prefix} = $word;
-                    $expects[$i]{word} = '';
-                    $expects[$i]{short_only} = 1;
-                    my $len_before_split = @words;
-                    my $j = $i+1;
-                  SHORTOPT:
-                    while ($shorts =~ s/(.)//) {
-                        $opt = "-$1";
-                        $opthash = $opts{$opt};
-                        if (!$opthash || $opthash->{parsed}{max_vals}) {
-                            # end after unknown short option or short option
-                            # expects value, and don't complete this optname
-                            # later
+                last unless $shorts =~ s/\A-([^-])(.*)/$2/;
+
+                my $opt = "-$1";
+                my $rest = $2;
+                my $opthash = $opts{$opt};
+                if (!$opthash) {
+                    # unknown option
+                    last SPLIT_BUNDLED;
+                }
+
+                $expects[$i]{short_only} = 1;
+                $expects[$i]{prefix} //= $word;
+
+                if ($opthash->{parsed}{max_vals}) {
+                    # option requires argument, slurp the rest as the option's
+                    # value
+                    $expects[$i]{word} = $rest;
+                    $expects[$i]{optval} = $opt;
+                    last SPLIT_BUNDLED;
+                }
+
+                my $len_before_split = @words;
+                my $j = $i+1;
+              SHORTOPT:
+                while ($shorts =~ s/(.)//) {
+                    $opt = "-$1";
+                    $opthash = $opts{$opt};
+
+                    if (!$opthash) {
+                        # we get -fz (where -f is a flag option and z is an
+                        # unknown option), we can't complete this so we return
+                        # an empty result.
+                        $expects[$i]{comp_result} = [];
+                        last SHORTOPT;
+                    } elsif ($opthash->{parsed}{max_vals}) {
+                        if (length $shorts) {
+                            # we get -fsblah (where -f is a flag option and -s
+                            # requires a value), let's complete this like we're
+                            # completing -s blah^ (completing option value)
+                            splice @words, $j, 0, $opt, '=', $shorts;
                             $expects[$i]{do_complete_optname} = 0;
-                            if (length $shorts) {
-                                splice @words, $j, 0, $opt, '=', $shorts;
-                                $j += 3;
-                            } else {
-                                splice @words, $j, 0, $opt;
-                                $j++;
-                            }
-                            last SHORTOPT;
+                            $expects[$i]{optval} = $opt;
+                            $expects[$i]{word} = $shorts;
+                            substr($expects[$i]{prefix},
+                               -length($shorts)) = '';
+                            $j += 3;
                         } else {
+                            # we get -fs (where -f is a flag option and -s
+                            # requires a value), let's complete this using
+                            # ['-fs'] so bash adds a space.
+                            $expects[$i]{comp_result} =
+                                [$expects[$i]{prefix}];
                             splice @words, $j, 0, $opt;
                             $j++;
-                            # continue splitting
                         }
+                        last SHORTOPT;
+                    } else {
+                        # we get another flag option
+                        $expects[$j]{prefix} = $expects[$i]{prefix};
+                        splice @words, $j, 0, $opt;
+                        $j++;
+                        # continue splitting
                     }
-                    $cword += @words-$len_before_split if $cword > $i;
-                    #say "D:words increases ", @words-$len_before_split;
                 }
+                $cword += @words-$len_before_split if $cword > $i;
+                #say "D:words increases ", @words-$len_before_split;
             }
 
             # split --foo=val -> --foo, =, val
@@ -410,7 +443,11 @@ sub complete_cli_arg {
                 $expects[$i]{optname} = $opt;
                 my $nth = $seen_opts{$opt} // 0;
                 $expects[$i]{nth} = $nth;
-                _mark_seen(\%seen_opts, $opt, \%opts);
+                for (defined($expects[$i]{prefix}) ?
+                         (map {"-$_"} split //, substr($expects[$i]{prefix}, 1)) :
+                             ($opt)) {
+                    _mark_seen(\%seen_opts, $_, \%opts);
+                }
 
                 my $min_vals = $opthash->{parsed}{min_vals};
                 my $max_vals = $opthash->{parsed}{max_vals};
@@ -478,6 +515,10 @@ sub complete_cli_arg {
         last unless exists $exp->{optname};
         last if defined($exp->{do_complete_optname}) &&
             !$exp->{do_complete_optname};
+        if ($exp->{comp_result}) {
+            push @answers, $exp->{comp_result};
+            last;
+        }
         my $opt = $exp->{optname};
         my @o;
         for (@optnames) {
@@ -537,10 +578,14 @@ sub complete_cli_arg {
         if ($comp) {
             $log->tracef("[comp][compgl] invoking routine supplied from 'completion' argument to complete option value, option=<%s>", $opt);
             $compres = $comp->(%compargs);
+            Complete::Util::modify_answer(answer=>$compres, prefix=>$exp->{prefix})
+                if defined $exp->{prefix};
             $log->tracef('[comp][compgl] adding result from routine: %s', $compres);
         }
         if (!$compres || !$comp) {
             $compres = _default_completion(%compargs);
+            Complete::Util::modify_answer(answer=>$compres, prefix=>$exp->{prefix})
+                if defined $exp->{prefix};
             $log->tracef('[comp][compgl] adding result from default '.
                              'completion routine');
         }
@@ -591,7 +636,7 @@ Complete::Getopt::Long - Complete command-line argument using Getopt::Long speci
 
 =head1 VERSION
 
-This document describes version 0.43 of Complete::Getopt::Long (from Perl distribution Complete-Getopt-Long), released on 2016-10-28.
+This document describes version 0.44 of Complete::Getopt::Long (from Perl distribution Complete-Getopt-Long), released on 2016-11-04.
 
 =head1 SYNOPSIS
 
@@ -743,7 +788,7 @@ Please visit the project's homepage at L<https://metacpan.org/release/Complete-G
 
 =head1 SOURCE
 
-Source repository is at L<https://github.com/perlancar/perl-Complete-Getopt-Long>.
+Source repository is at L<https://github.com/sharyanto/perl-Complete-Getopt-Long>.
 
 =head1 BUGS
 
